import logging

import numpy as np

from core.constants import CellValue
from core.state import World
from tools.perlin import generate_fractal_noise_2d
from ui import UserInterface, Theme
from ui.mode import EditGameMode

logging.basicConfig(level=logging.INFO, format='\r%(asctime)s %(filename)s:%(lineno)d: %(message)s')


# Create a basic game state
world = World((1024, 1024))
ground = world.ground.cells
np.random.seed(0)
noise = generate_fractal_noise_2d(world.size, (32, 32), 5)
ground[noise > noise.mean()] = CellValue.GROUND_EARTH

# Create a user interface and run it
theme = Theme()
tileSize = theme.getTileset("ground").tileSize
theme.viewSize = (20 * tileSize[0], 15 * tileSize[1])
userInterface = UserInterface(theme)
gameMode = EditGameMode(theme, world)
userInterface.setGameMode(gameMode)
userInterface.run()
userInterface.quit()
