from typing import Optional, Union

from core.constants import CellValueRanges
from core.constants.CellValue import getCellValues
from core.state import World
from .FrameComponent import FrameComponent
from ..control.Button import Button
from ...Mouse import Mouse
from ...theme.Theme import Theme


class PaletteFrame(FrameComponent):

    def __init__(self, theme: Theme, world: World, rowCount: int = 2):
        super().__init__(theme)

        button: Optional[Button] = None
        for name in world.layerNames:
            button = self.__addTilesetButtons(name, rowCount, button)

        self.pack()

    def __createButton(self, layerName: str,
                       tileId: Union[str, int]) -> Button:
        def buttonAction(mouse: Mouse):
            if mouse.button1:
                self.notifyMainBrushSelected(layerName, tileId)
            elif mouse.button3:
                self.notifySecondaryBrushSelected(layerName, tileId)
        tileset = self.theme.getTileset(layerName)
        tile = tileset.getTile(tileId)
        return Button(self.theme, tile, buttonAction)

    def __addTilesetButtons(self, layerName: str, rowCount: int,
                            columnButton: Optional[Button] = None) -> Optional[Button]:
        tileset = self.theme.getTileset(layerName)
        previousButton: Optional[Button] = None
        columnIndex = 0
        for tileId in getCellValues(layerName):
            button = self.__createButton(layerName, tileId)
            if columnButton is None:
                columnButton = button
                button.moveRelativeTo("topLeft", self, "topLeft", borderSize=0)
            elif columnIndex == 0:
                button.moveRelativeTo("left", columnButton, "right", borderSize=0)
            else:
                button.moveRelativeTo("top", previousButton, "bottom", borderSize=0)
            self.addComponent(button)
            previousButton = button
            if columnIndex == 0:
                columnButton = button
            columnIndex += 1
            if columnIndex >= rowCount:
                columnIndex = 0

        return columnButton
