from typing import Tuple, Optional

import numpy as np

from core.constants import CellValue


class LayerRenderer:

    def __init__(self, cells: np.ndarray,
                 cellMinX: int, cellMinY: int, cellMaxX: int, cellMaxY: int,
                 view: Tuple[int, int], tileSize: Tuple[int, int]):
        self.cells = cells
        self.view = view
        self.tileSize = tileSize
        self.cellsBox = (cellMinX, cellMaxX, cellMinY, cellMaxY)
        self.cellsSlice = np.s_[cellMinX:cellMaxX, cellMinY:cellMaxY]

    def coords(self, validCells: Optional[np.ndarray] = None):
        tileWidth, tileHeight = self.tileSize
        viewX, viewY = self.view
        if validCells is None:
            cells = self.cells
            cellMinX, cellMaxX, cellMinY, cellMaxY = self.cellsBox
            shiftX = cellMinX * tileWidth + viewX % tileWidth
            shiftY = cellMinY * tileHeight + viewY % tileHeight
            for cellY in range(cellMinY, cellMaxY):
                for cellX in range(cellMinX, cellMaxX):
                    value = CellValue(cells[cellX, cellY])
                    destX = cellX * tileWidth - shiftX
                    destY = cellY * tileHeight - shiftY
                    yield (destX, destY), value, (cellX - cellMinX, cellY - cellMinY)
        else:
            cells = self.cells[self.cellsSlice]
            cellCoords = np.transpose(np.nonzero(validCells))
            for cell in cellCoords:
                cellRelX = int(cell[0])
                cellRelY = int(cell[1])
                value = CellValue(cells[cellRelX, cellRelY])
                destX = cellRelX * tileWidth - viewX % tileWidth
                destY = cellRelY * tileHeight - viewY % tileHeight
                yield (destX, destY), value, (cellRelX, cellRelY)
                